#ifndef EID_VWR_EID_VIEWER_H
#define EID_VWR_EID_VIEWER_H

/** \addtogroup C_API
  *
  * @{
  */

/** \file eid-viewer.h
  * \brief Main include file. */

#include <eid-viewer/oslayer.h>
#ifndef WIN32
#include <eid-viewer/verify_cert.h>
#include <eid-viewer/certhelpers.h>
#endif

/**@}*/

#endif
